#pragma once

//Copyright (C) cocosimulator.org 2014
//generated by AmsterCHEM's COM CAPE-OPEN Wizard

// Basic COM object

class COMObject : public IUnknown {
public:


	//derived classes that implement interfaces must implement InternalQueryInterface
	// InternalQueryInterface will return a class pointer if succesful.

	virtual void *InternalQueryInterface(REFIID riid)=0;

	// IUnknown
	virtual STDMETHODIMP_(ULONG) AddRef(void) {
		//non thread-safe version; we are an appartment threaded DLL
		m_cRef++;
		return m_cRef;
	}

	virtual STDMETHODIMP_(ULONG) Release(void) {
		m_cRef--;
		if (m_cRef==0) {
			delete this;
			return 0;
		}
		return 1;
	}

	STDMETHOD(QueryInterface)(REFIID riid, void **ppv) {
		if (ppv == NULL) {
			return E_INVALIDARG;
		}
		if (riid==IID_IUnknown) {
			*ppv=(IUnknown*)this;
		} else {
			*ppv=InternalQueryInterface(riid);    //check derived object
		}
		if (!*ppv) {
			return E_NOINTERFACE;
		}
		AddRef();
		return S_OK;
	}

	// constructors/destructors
	// a COM object keeps track of its own reference count (do not delete, use Release instead)
	// it will also keep track of the global COM object count (see DllCanUnloadNow)

	COMObject() {
		m_cRef = 1;
		InterlockedIncrement(&ModuleLockCount);
	}

	virtual ~COMObject() {
		InterlockedDecrement(&ModuleLockCount);
	}

private:

	LONG m_cRef; //reference count; this one is not accessed from multiple threads.

};

//macros to implement the IUnknown and IDispatch+IUnknown interfaces on classes derived from COMROOT
// as we do not expect CAPE-OPEN (multi-interface objects) to be used by
// late binding (which according to microsoft should not be done on multi-interfaced objects)
// but we need to implement it (as CAPE-OPEN interfaces derive from IDispatch)
// we provide a dummy - non-functional - implementation.
// HENCE: LATE BINDING WILL NOT WORK

#define IMPLEMENT_IUNKNOWN \
	virtual STDMETHODIMP_(ULONG) AddRef(void){return COMObject::AddRef();}\
	virtual STDMETHODIMP_(ULONG) Release(void) {return COMObject::Release();}\
	STDMETHOD(QueryInterface)(REFIID riid, void **ppv) {return COMObject::QueryInterface(riid,ppv);}
#define IMPLEMENT_IDISPATCH \
	IMPLEMENT_IUNKNOWN;\
	STDMETHOD(GetTypeInfoCount)(unsigned int FAR*i) {*i=0;return NO_ERROR;} \
	STDMETHOD(GetTypeInfo)(unsigned int ,LCID,ITypeInfo FAR* FAR*){return TYPE_E_ELEMENTNOTFOUND;} \
	STDMETHOD(GetIDsOfNames)(REFIID,OLECHAR FAR* FAR*,unsigned int,LCID,DISPID FAR*){return DISP_E_UNKNOWNNAME;} \
	STDMETHOD(Invoke)(DISPID,REFIID,LCID,WORD,DISPPARAMS FAR*,VARIANT FAR*,EXCEPINFO FAR*,unsigned int FAR*){return E_NOTIMPL;}
