#pragma once
#include <stdlib.h>

//Copyright (C) cocosimulator.org 2014
//generated by AmsterCHEM's COM CAPE-OPEN Wizard

//a collection of string conversion classes. Can be used as if they are a function.

#define STRINGCONVERSION_STACKVARIABLELENGTH 128

class MultibyteToUnicode {
	OLECHAR stack[STRINGCONVERSION_STACKVARIABLELENGTH];
	OLECHAR *target;

	public:

	MultibyteToUnicode(const char *str) {
		size_t len=lstrlenA(str);
		if (len<STRINGCONVERSION_STACKVARIABLELENGTH) {
			//use stack
			#ifdef _MSC_VER
			#pragma warning( push ) 
			#pragma warning( disable : 4996) 
			#endif
			mbstowcs(stack,str,STRINGCONVERSION_STACKVARIABLELENGTH);
			#ifdef _MSC_VER
			#pragma warning( pop ) 
			#endif
			target=stack;
		} else {
			//use heap
			len++;
			target=new OLECHAR[len];
			#ifdef _MSC_VER
			#pragma warning( push ) 
			#pragma warning( disable : 4996) 
			#endif
			mbstowcs(target,str,len);
			#ifdef _MSC_VER
			#pragma warning( pop ) 
			#endif
		}
	}

	MultibyteToUnicode(const std::string &str) {
		if (str.size()<STRINGCONVERSION_STACKVARIABLELENGTH) {
			//use stack
			#ifdef _MSC_VER
			#pragma warning( push ) 
			#pragma warning( disable : 4996) 
			#endif
			mbstowcs(stack,str.c_str(),STRINGCONVERSION_STACKVARIABLELENGTH);
			#ifdef _MSC_VER
			#pragma warning( pop ) 
			#endif
			target=stack;
		} else {
			//use heap
			size_t len=str.size();
			len++;
			target=new OLECHAR[len];
			#ifdef _MSC_VER
			#pragma warning( push ) 
			#pragma warning( disable : 4996) 
			#endif
			mbstowcs(target,str.c_str(),len);
			#ifdef _MSC_VER
			#pragma warning( pop ) 
			#endif
		}
	}

	~MultibyteToUnicode() {
		if (stack!=target) delete []target;
	}

	operator const OLECHAR *() {return target;}

};

class UnicodeToMultibyte{
	char stack[STRINGCONVERSION_STACKVARIABLELENGTH];
	char *target;

	public:

	UnicodeToMultibyte(const std::wstring &str) {
		if (str.size()<(STRINGCONVERSION_STACKVARIABLELENGTH>>1)) {
			//use stack, the result will fit 
			#ifdef _MSC_VER
			#pragma warning( push ) 
			#pragma warning( disable : 4996) 
			#endif
			wcstombs(stack,str.c_str(),STRINGCONVERSION_STACKVARIABLELENGTH);
			#ifdef _MSC_VER
			#pragma warning( pop ) 
			#endif
			target=stack;
		} else {
			//use heap, the result may not fit
			size_t len=str.size();
			len<<=1; 
			len++;
			target=new char[len];
			#ifdef _MSC_VER
			#pragma warning( push ) 
			#pragma warning( disable : 4996) 
			#endif
			wcstombs(target,str.c_str(),len);
			#ifdef _MSC_VER
			#pragma warning( pop ) 
			#endif
		}
	}

	UnicodeToMultibyte(const OLECHAR *str) {
		size_t len=lstrlenW(str);
		if (len<(STRINGCONVERSION_STACKVARIABLELENGTH>>1)) {
			//use stack, the result will fit 
			#ifdef _MSC_VER
			#pragma warning( push ) 
			#pragma warning( disable : 4996) 
			#endif
			wcstombs(stack,str,STRINGCONVERSION_STACKVARIABLELENGTH);
			#ifdef _MSC_VER
			#pragma warning( pop ) 
			#endif
			target=stack;
		} else {
			//use heap, the result may not fit
			len<<=1; 
			len++;
			target=new char[len];
			#ifdef _MSC_VER
			#pragma warning( push ) 
			#pragma warning( disable : 4996) 
			#endif
			wcstombs(target,str,len);
			#ifdef _MSC_VER
			#pragma warning( pop ) 
			#endif
		}
	}

	~UnicodeToMultibyte() {
		if (target!=stack) delete []target;
	}

	operator const char *() {return target;}
};

//these are for out-of-place conversion. For in-place conversion
// such as inside a wstring, use std::transform instead

template <int Convert(int c)> class StringConversion {
	
	OLECHAR stack[STRINGCONVERSION_STACKVARIABLELENGTH];
	OLECHAR *target;

	public:

	StringConversion(const OLECHAR *str) {
		const OLECHAR *source=str;
		const OLECHAR *end=str+STRINGCONVERSION_STACKVARIABLELENGTH-1;
		OLECHAR *tgt;
		tgt=target=stack;
		while (*source) {
			if (source==end) {
				//won't fit, use heap
				int len=lstrlenW(source)+STRINGCONVERSION_STACKVARIABLELENGTH;
				target=new OLECHAR[len];
				memcpy(target,stack,(STRINGCONVERSION_STACKVARIABLELENGTH-1)<<1);
				tgt=target+STRINGCONVERSION_STACKVARIABLELENGTH-1;
				while (*(source)) *(tgt++)=Convert(*(source++));
				break;
			}
			*(tgt++)=Convert(*(source++));
		}
		*tgt=0;
	}

	StringConversion(const std::wstring &str) {
		if (str.size()>=STRINGCONVERSION_STACKVARIABLELENGTH) {
			target=new OLECHAR[str.size()+1];
		} else {
			target=stack;
		}
		OLECHAR *tgt=target;
		const OLECHAR *source=str.c_str();
		while (*source) *(tgt++)=Convert(*(source++));
		*tgt=0;
	}

	~StringConversion() {
		if (target!=stack) delete []target;
	}

	operator const OLECHAR *() {return target;}

};

typedef StringConversion<tolower> MakeLowerCase;
typedef StringConversion<toupper> MakeUpperCase;

//a string class with additional functions

#include <ctype.h>

#ifdef __GNUC__
//GCC does not like the _stdcall functions
#define STRFUNCTYPE _cdecl
#define STRLENTYPE size_t
#else
#define STRFUNCTYPE WINAPI
#define STRLENTYPE int
#endif

template <class STRING,class CHAR,int VSNPRINTF(CHAR *buffer,size_t count,const CHAR *format,va_list argptr),int STRFUNCTYPE STRCMP(const CHAR*,const CHAR*),int STRFUNCTYPE STRCMPI(const CHAR*,const CHAR*),STRLENTYPE STRFUNCTYPE STRLEN(const CHAR*)> class StringEnh : public STRING {
	public:

	//some constructors
	StringEnh() {}
	StringEnh(const CHAR *str) : STRING(str) {}
	StringEnh(const STRING &str) : STRING(str) {}
	StringEnh(const StringEnh &str) : STRING(*(STRING*)&str) {}

	//overload some operators

	const StringEnh &operator = (const CHAR * str) {*(STRING*)this=str;return *this;}
	const StringEnh &operator = (const STRING &str) {*(STRING*)this=str;return *this;}
	const StringEnh &operator = (const StringEnh &str) {*(STRING*)this=*(STRING*)&str;return *this;}
	const StringEnh &operator += (const StringEnh & str) {(*this)+=*(STRING*)&str;return *this;}
	const StringEnh &operator += (const CHAR *str) {*(STRING*)this+=str;return *this;}
	const StringEnh &operator += (const STRING &str) {*(STRING*)this+=str;return *this;}
	const StringEnh &operator += (const CHAR c) {*(STRING*)this+=c;return *this;}

	//formatting

	void printf(const CHAR *fmt, ...) {
		//printf, we grow the buffer until it fits
		Locker lock; //we use a static buffer
		static vector<CHAR> buffer; //vector is garantueed to be layed out flat in memory; string is not
		if (buffer.size()<128) {
			buffer.resize(128);
			buffer.resize(buffer.capacity());
		}
		va_list marker;
		for (;;) {
			va_start(marker, fmt);
			#ifdef _MSC_VER
			#pragma warning( push ) 
			#pragma warning( disable : 4996) 
			#endif
			int res=VSNPRINTF((CHAR*)buffer.data(),buffer.size()-1, fmt, marker);
			#ifdef _MSC_VER
			#pragma warning( pop ) 
			#endif
			va_end(marker);
			if (res>=0) {
				(*(STRING*)this).assign(buffer.data(),res);
				break;
			}
			//try again
			buffer.resize(buffer.size()*2);
			buffer.resize(buffer.capacity()); 
		}
	}

	//comparison

	int strcmp(const CHAR*str) {return STRCMP((*(STRING*)this).c_str(),str);}
	int strcmpi(const CHAR*str) {return STRCMPI((*(STRING*)this).c_str(),str);}
	int strcmp(const STRING &str) {return STRCMP((*(STRING*)this).c_str(),str.c_str());}
	int strcmpi(const STRING &str) {return STRCMPI((*(STRING*)this).c_str(),str.c_str());}
	int strcmp(const StringEnh &str) {return STRCMP((*(STRING*)this).c_str(),str.c_str());}
	int strcmpi(const StringEnh &str) {return STRCMPI((*(STRING*)this).c_str(),str.c_str());}

	//in-place case conversion

	void MakeUpperCase() {
		transform((*(STRING*)this).begin(),(*(STRING*)this).end(),(*(STRING*)this).begin(),::toupper);
	}
	void MakeLowerCase() {
		transform((*(STRING*)this).begin(),(*(STRING*)this).end(),(*(STRING*)this).begin(),::tolower);
	}

	//trimming white space (space + tab)

	void TrimLeft() {
		size_t index=0;
		while (index<(*(STRING*)this).size()) {
			if (((*this)[index]!=' ')&&((*this)[index]!='\t')) break;
			index++;
		}
		if (index) (*this)=(*(STRING*)this).substr(index);
	}

	void TrimRight() {
		size_t index=(*(STRING*)this).size();
		while (index) {
			if (((*this)[index-1]!=' ')&&((*this)[index-1]!='\t')) break;
			index--;
		}
		if (index<(*(STRING*)this).size()) (*(STRING*)this).resize(index);
	}
	
	void Trim() {
		TrimLeft();
		TrimRight();
	}

	bool beginswithi(const CHAR *txt) {
		//insensitive check for initial text
		int len=STRLEN(txt);
		if ((int)(*(STRING*)this).size()<len) return false;
		for (int i=0;i<len;i++)	
			if (::tolower(txt[i])!=::tolower((*this)[i]))
				return false;
		return true;
	}

};

#ifdef __GNUC__
//GCC does not like the _stdcall functions
typedef StringEnh<std::string,char,vsnprintf,strcmp,strcmpi,strlen> stringenh;
typedef StringEnh<std::wstring,wchar_t,_vsnwprintf,wcscmp,wcsicmp,wcslen> wstringenh;
#else
typedef StringEnh<std::string,char,vsnprintf,lstrcmpA,lstrcmpiA,lstrlenA> stringenh;
typedef StringEnh<std::wstring,wchar_t,_vsnwprintf,lstrcmpW,lstrcmpiW,lstrlenW> wstringenh;
#endif

#undef STRINGCONVERSION_STACKVARIABLELENGTH